@@ -8,78 +8,77 @@
 #include "defaultIni.hpp"
 #include "ankerl_unordered_dense.h"
 
+#if !defined(__EMSCRIPTEN__)
+    #include <curl/curl.h>
+    #include <curl/easy.h>
+#endif
+
 
 namespace Themes {
 
+
     EXPORT BaseTheme::BaseTheme() {
 
         fcCoverage.setStyle(SkPaint::kStrokeAndFill_Style);
         fcCoverage.setStrokeWidth(0);
-//        fcCoverage.setAntiAlias(true);
 
-        std::vector<std::vector<int>> tmp = {{158, 1,   66},
-                                             {179, 24,  71},
-                                             {203, 51,  76},
-                                             {220, 73,  75},
-                                             {233, 92,  71},
-                                             {244, 114, 69},
-                                             {248, 142, 82},
-                                             {252, 167, 94},
-                                             {253, 190, 110},
-                                             {253, 212, 129},
-                                             {254, 228, 147},
-                                             {254, 242, 169},
-                                             {254, 254, 190},
-                                             {244, 250, 174},
-                                             {233, 246, 158},
-                                             {213, 238, 155},
-                                             {190, 229, 160},
-                                             {164, 218, 164},
-                                             {134, 206, 164},
-                                             {107, 196, 164},
-                                             {83,  173, 173},
-                                             {61,  148, 183},
-                                             {57,  125, 184},
-                                             {76,  101, 172},
-                                             {94,  79,  162},
-                                             {255, 255, 229},
-                                             {252, 254, 215},
-                                             {249, 253, 200},
-                                             {246, 251, 184},
-                                             {237, 248, 178},
-                                             {227, 244, 170},
-                                             {216, 239, 162},
-                                             {202, 233, 156},
-                                             {187, 227, 149},
-                                             {172, 220, 141},
-                                             {155, 213, 135},
-                                             {137, 205, 127},
-                                             {119, 197, 120},
-                                             {101, 189, 111},
-                                             {82,  179, 102},
-                                             {64,  170, 92},
-                                             {55,  158, 84},
-                                             {44,  144, 75},
-                                             {34,  131, 66},
-                                             {23,  122, 62},
-                                             {11,  112, 58},
-                                             {0,   103, 54},
-                                             {0,   92,  50},
-                                             {0,   79,  45},
-                                             {0,   69,  41}};
-        for (size_t i = 0; i < tmp.size(); ++i) {
-            SkPaint p;
-            p.setARGB(255, tmp[i][0], tmp[i][1], tmp[i][2]);
-            mate_fc.push_back(p);
-        }
+        // Set ARGB values directly for each SkPaint object in the mate_fc vector
+        mate_fc[0].setARGB(255, 158, 1, 66);
+        mate_fc[1].setARGB(255, 179, 24, 71);
+        mate_fc[2].setARGB(255, 203, 51, 76);
+        mate_fc[3].setARGB(255, 220, 73, 75);
+        mate_fc[4].setARGB(255, 233, 92, 71);
+        mate_fc[5].setARGB(255, 244, 114, 69);
+        mate_fc[6].setARGB(255, 248, 142, 82);
+        mate_fc[7].setARGB(255, 252, 167, 94);
+        mate_fc[8].setARGB(255, 253, 190, 110);
+        mate_fc[9].setARGB(255, 253, 212, 129);
+        mate_fc[10].setARGB(255, 254, 228, 147);
+        mate_fc[11].setARGB(255, 254, 242, 169);
+        mate_fc[12].setARGB(255, 254, 254, 190);
+        mate_fc[13].setARGB(255, 244, 250, 174);
+        mate_fc[14].setARGB(255, 233, 246, 158);
+        mate_fc[15].setARGB(255, 213, 238, 155);
+        mate_fc[16].setARGB(255, 190, 229, 160);
+        mate_fc[17].setARGB(255, 164, 218, 164);
+        mate_fc[18].setARGB(255, 134, 206, 164);
+        mate_fc[19].setARGB(255, 107, 196, 164);
+        mate_fc[20].setARGB(255, 83, 173, 173);
+        mate_fc[21].setARGB(255, 61, 148, 183);
+        mate_fc[22].setARGB(255, 57, 125, 184);
+        mate_fc[23].setARGB(255, 76, 101, 172);
+        mate_fc[24].setARGB(255, 94, 79, 162);
+        mate_fc[25].setARGB(255, 255, 255, 229);
+        mate_fc[26].setARGB(255, 252, 254, 215);
+        mate_fc[27].setARGB(255, 249, 253, 200);
+        mate_fc[28].setARGB(255, 246, 251, 184);
+        mate_fc[29].setARGB(255, 237, 248, 178);
+        mate_fc[30].setARGB(255, 227, 244, 170);
+        mate_fc[31].setARGB(255, 216, 239, 162);
+        mate_fc[32].setARGB(255, 202, 233, 156);
+        mate_fc[33].setARGB(255, 187, 227, 149);
+        mate_fc[34].setARGB(255, 172, 220, 141);
+        mate_fc[35].setARGB(255, 155, 213, 135);
+        mate_fc[36].setARGB(255, 137, 205, 127);
+        mate_fc[37].setARGB(255, 119, 197, 120);
+        mate_fc[38].setARGB(255, 101, 189, 111);
+        mate_fc[39].setARGB(255, 82, 179, 102);
+        mate_fc[40].setARGB(255, 64, 170, 92);
+        mate_fc[41].setARGB(255, 55, 158, 84);
+        mate_fc[42].setARGB(255, 44, 144, 75);
+        mate_fc[43].setARGB(255, 34, 131, 66);
+        mate_fc[44].setARGB(255, 23, 122, 62);
+        mate_fc[45].setARGB(255, 11, 112, 58);
+        mate_fc[46].setARGB(255, 0, 103, 54);
+        mate_fc[47].setARGB(255, 0, 92, 50);
+        mate_fc[48].setARGB(255, 0, 79, 45);
+        mate_fc[49].setARGB(255, 0, 69, 41);
 
         ecMateUnmapped.setARGB(255, 255, 0, 0);
         ecMateUnmapped.setStyle(SkPaint::kStroke_Style);
         ecMateUnmapped.setStrokeWidth(1);
 
         fcIns.setStyle(SkPaint::kFill_Style);
-        //fcIns.setARGB(255, 178, 77, 255);
-
 
         lwMateUnmapped = 0.5;
         lwSplit = 0.5;
@@ -111,7 +110,7 @@ namespace Themes {
         fcSoftClip0 = fcSoftClip;
 
         bgPaint.setStyle(SkPaint::kStrokeAndFill_Style);
-        bgPaint.setStrokeWidth(2);
+        bgPaint.setStrokeWidth(0);
 
         fcNormal.setAlpha(alpha);
 
@@ -124,7 +123,7 @@ namespace Themes {
         fcSoftClip0.setAlpha(mapq0_alpha);
 
         fcTrack.setStyle(SkPaint::kStrokeAndFill_Style);
-        fcTrack.setStrokeWidth(2);
+        fcTrack.setStrokeWidth(0);
         fcTrack.setAntiAlias(true);
 
         lcJoins.setStyle(SkPaint::kStroke_Style);
@@ -141,6 +140,9 @@ namespace Themes {
         lcBright.setStrokeWidth(2);
         lcBright.setAntiAlias(true);
 
+        ecSelected.setAntiAlias(true);
+        ecSplit.setAntiAlias(true);
+
         fcMarkers.setStyle(SkPaint::kStrokeAndFill_Style);
         fcMarkers.setAntiAlias(true);
         fcMarkers.setStrokeMiter(0.1);
@@ -150,12 +152,17 @@ namespace Themes {
         fc5mc.setStrokeCap(SkPaint::kRound_Cap);
         fc5hmc.setAntiAlias(true);
         fc5hmc.setStrokeCap(SkPaint::kRound_Cap);
+        fcOther.setAntiAlias(true);
+        fcOther.setStrokeCap(SkPaint::kRound_Cap);
+        fcOther.setARGB(127, 255, 0, 0);
 
+        mate_fc0 = mate_fc;
         for (size_t i=0; i < mate_fc.size(); ++i) {
-            SkPaint p = mate_fc[i];
+//            SkPaint p = mate_fc[i];
             mate_fc[i].setAlpha(alpha);
-            p.setAlpha(mapq0_alpha);
-            mate_fc0.push_back(p);
+            mate_fc0[i].setAlpha(mapq0_alpha);
+//            p.setAlpha(mapq0_alpha);
+//            mate_fc0.emplace_back(p);
         }
         SkPaint p;
         // A==1, C==2, G==4, T==8, N==>8
@@ -186,11 +193,25 @@ namespace Themes {
             BasePaints[14][i] = p;
             BasePaints[15][i] = p;
         }
+        int alph = 63;
+        for (size_t i=0; i < 4; ++i) {
+            ModPaints[0][i] = fc5mc;
+            ModPaints[0][i].setAlpha(alph);
+
+            ModPaints[1][i] = fc5hmc;
+            ModPaints[1][i].setAlpha(alph);
+
+            ModPaints[2][i] = fcOther;
+            ModPaints[2][i].setAlpha(alph);
+
+            alph += 64;
+        }
     }
 
     void BaseTheme::setPaintARGB(int paint_enum, int a, int r, int g, int b) {
         switch (paint_enum) {
             case GwPaint::bgPaint: this->bgPaint.setARGB(a, r, g, b); break;
+            case GwPaint::bgPaintTiled: this->bgPaintTiled.setARGB(a, r, g, b); break;
             case GwPaint::fcNormal: this->fcNormal.setARGB(a, r, g, b); break;
             case GwPaint::fcDel: this->fcDel.setARGB(a, r, g, b); break;
             case GwPaint::fcDup: this->fcDup.setARGB(a, r, g, b); break;
@@ -204,6 +225,9 @@ namespace Themes {
             case GwPaint::fcC: this->fcC.setARGB(a, r, g, b); break;
             case GwPaint::fcG: this->fcG.setARGB(a, r, g, b); break;
             case GwPaint::fcN: this->fcN.setARGB(a, r, g, b); break;
+            case GwPaint::fc5mc: this->fc5mc.setARGB(a, r, g, b); break;
+            case GwPaint::fc5hmc: this->fc5hmc.setARGB(a, r, g, b); break;
+            case GwPaint::fcOther: this->fcOther.setARGB(a, r, g, b); break;
             case GwPaint::fcCoverage: this->fcCoverage.setARGB(a, r, g, b); break;
             case GwPaint::fcTrack: this->fcTrack.setARGB(a, r, g, b); break;
             case GwPaint::fcNormal0: this->fcNormal0.setARGB(a, r, g, b); break;
@@ -235,16 +259,17 @@ namespace Themes {
     EXPORT IgvTheme::IgvTheme() {
         name = "igv";
         fcCoverage.setARGB(255, 195, 195, 195);
-        fcTrack.setARGB(200, 0, 0, 0);
+        fcTrack.setARGB(200, 20, 20, 20);
         bgPaint.setARGB(255, 255, 255, 255);
-        bgMenu.setARGB(255, 250, 250, 250);
+        bgPaintTiled.setARGB(255, 235, 235, 235);
+        bgMenu.setARGB(255, 248, 248, 246);
         fcNormal.setARGB(255, 202, 202, 202);
         fcDel.setARGB(255, 225, 19, 67);
         fcDup.setARGB(255, 0, 54, 205);
         fcInvF.setARGB(255, 0, 199, 50);
         fcInvR.setARGB(255, 0, 199, 50);
         fcTra.setARGB(255, 255, 105, 180);
-        fcSoftClip.setARGB(255, 0, 128, 128);
+        fcSoftClip.setARGB(255, 96,160,176);
         fcA.setARGB(255, 109, 230, 64);
         fcT.setARGB(255, 255, 0, 107);
         fcC.setARGB(255, 66, 127, 255);
@@ -252,9 +277,10 @@ namespace Themes {
         fcN.setARGB(255, 128, 128, 128);
         fcIns.setARGB(255, 158, 112, 250);
 
-        fc5mc.setARGB(127, 194, 151, 58);
-//        fc5hmc.setARGB(127, 52, 255, 96);
-        fc5hmc.setARGB(127, 189, 78, 23);
+        fc5mc.setARGB(255, 155, 20, 15);
+//        fc5mc.setARGB(227, 194, 151, 58);
+//        fc5hmc.setARGB(227, 189, 78, 23);
+        fc5hmc.setARGB(255, 10, 100, 250);
         lcJoins.setARGB(255, 80, 80, 80);
         lcLightJoins.setARGB(255, 140, 140, 140);
         lcLabel.setARGB(255, 80, 80, 80);
@@ -277,7 +303,8 @@ namespace Themes {
         fcCoverage.setARGB(255, 95, 95, 105);
         fcTrack.setARGB(200, 227, 232, 255);
         bgPaint.setARGB(255, 10, 10, 20);
-        bgMenu.setARGB(255, 20, 20, 30);
+        bgPaintTiled.setARGB(255, 40, 40, 40);
+        bgMenu.setARGB(255, 30, 30, 40);
         fcNormal.setARGB(255, 90, 90, 95);
         fcDel.setARGB(255, 185, 25, 25);
         fcDup.setARGB(255, 24, 100, 198);
@@ -292,9 +319,9 @@ namespace Themes {
         fcN.setARGB(255, 128, 128, 128);
         fcIns.setARGB(255, 158, 112, 250);
 //        fc5mc.setARGB(127, 252, 186, 3);
-        fc5mc.setARGB(127, 30, 176, 230);
+        fc5mc.setARGB(227, 30, 176, 230);
 //        fc5hmc.setARGB(127, 52, 255, 96);
-        fc5hmc.setARGB(127, 215, 85, 23);
+        fc5hmc.setARGB(227, 215, 85, 23);
         lcJoins.setARGB(255, 142, 142, 142);
         lcLightJoins.setARGB(255, 82, 82, 82);
         lcLabel.setARGB(255, 182, 182, 182);
@@ -317,6 +344,7 @@ namespace Themes {
         fcCoverage.setARGB(255, 103, 102, 109);
         fcTrack.setARGB(200, 227, 232, 255);
         bgPaint.setARGB(255, 45, 45, 48);
+        bgPaintTiled.setARGB(255, 25, 25, 28);
         bgMenu.setARGB(255, 0, 0, 0);
         fcNormal.setARGB(255, 93, 92, 99);
         fcDel.setARGB(255, 185, 25, 25);
@@ -332,9 +360,9 @@ namespace Themes {
         fcG.setARGB(255, 226, 132, 19);
         fcN.setARGB(255, 128, 128, 128);
 //        fc5mc.setARGB(127, 252, 186, 3);
-        fc5mc.setARGB(127, 30, 176, 230);
+        fc5mc.setARGB(227, 30, 176, 230);
 //        fc5hmc.setARGB(127, 52, 255, 96);
-        fc5hmc.setARGB(127, 215, 85, 23);
+        fc5hmc.setARGB(227, 215, 85, 23);
         lcJoins.setARGB(255, 142, 142, 142);
         lcLightJoins.setARGB(255, 82, 82, 82);
         lcLabel.setARGB(255, 182, 182, 182);
@@ -358,7 +386,7 @@ namespace Themes {
         theme_str = "dark";
         dimensions_str = "1366x768";
         dimensions = {1366, 768};
-        link = "None";
+        link = "none";
         link_op = 0;
         number_str = "3x3";
         number = {3, 3};
@@ -383,7 +411,7 @@ namespace Themes {
         variant_distance = 100000;
         low_memory = 1500000;
 
-        max_coverage = 10000000;
+        max_coverage = 100000;
         max_tlen = 2000;
 
         editing_underway = false;
@@ -392,12 +420,13 @@ namespace Themes {
         tlen_yscale = false;
         expand_tracks = false;
         vcf_as_tracks = false;
-        bed_as_tracks = true;
+        bed_as_tracks = false;
         sv_arcs = true;
-        parse_mods = true;
+        parse_mods = false;
+        scale_bar = true;
 
         scroll_speed = 0.15;
-        tab_track_height = 0.05;
+        tab_track_height = 0.25;
         scroll_right = GLFW_KEY_RIGHT;
         scroll_left = GLFW_KEY_LEFT;
         scroll_down = GLFW_KEY_PAGE_DOWN;
@@ -407,9 +436,6 @@ namespace Themes {
         zoom_out = GLFW_KEY_DOWN;
         zoom_in = GLFW_KEY_UP;
         cycle_link_mode = GLFW_KEY_L;
-        print_screen = GLFW_KEY_PRINT_SCREEN;
-        delete_labels = GLFW_KEY_DELETE;
-        enter_interactive_mode = GLFW_KEY_ENTER;
         find_alignments=GLFW_KEY_F;
         repeat_command=GLFW_KEY_R;
     }
@@ -430,13 +456,16 @@ namespace Themes {
 
         std::unordered_map<std::string, int> key_table;
         Keys::getKeyTable(key_table);
+        bool update_ini = false;
 
-        theme_str = myIni["general"]["theme"];
+        mINI::INIMap<std::string>& sub = myIni["general"];
+
+        theme_str = sub["theme"];
         if (theme_str == "dark") { theme = Themes::DarkTheme(); } else if (theme_str == "slate") { theme = Themes::SlateTheme(); } else { theme = Themes::IgvTheme(); }
-        dimensions_str = myIni["general"]["dimensions"];
+        dimensions_str = sub["dimensions"];
         dimensions = Utils::parseDimensions(dimensions_str);
 
-        link = myIni["general"]["link"];
+        link = sub["link"];
         link_op = 0;
         if (link == "sv") {
             link_op = 1;
@@ -446,102 +475,215 @@ namespace Themes {
             link = "none";
         }
 
-        indel_length = std::stoi(myIni["general"]["indel_length"]);
-        ylim = std::stoi(myIni["general"]["ylim"]);
-        split_view_size = std::stoi(myIni["general"]["split_view_size"]);
-        threads = std::stoi(myIni["general"]["threads"]);
-        pad = std::stoi(myIni["general"]["pad"]);
-        max_coverage = (myIni["general"]["coverage"] == "true") ? 100000 : 0;
-        log2_cov = myIni["general"]["log2_cov"] == "true";
-        scroll_speed = std::stof(myIni["general"]["scroll_speed"]);
-        tab_track_height = std::stof(myIni["general"]["tabix_track_height"]);
-        if (myIni["general"].has("font")) {
-            font_str = myIni["general"]["font"];
-        }
-        if (myIni["general"].has("font_size")) {
-            font_size = std::stoi(myIni["general"]["font_size"]);
-        }
-        if (myIni["general"].has("expand_tracks")) {
-            expand_tracks = myIni["general"]["expand_tracks"] == "true";
-        }
-        if (myIni["general"].has("sv_arcs")) {
-            sv_arcs = myIni["general"]["sv_arcs"] == "true";
-        }
-        if (myIni["general"].has("mods")) {
-            parse_mods = myIni["general"]["mods"] == "true";
-        }
-        if (myIni["general"].has("session_file")) {
-            session_file = myIni["general"]["session_file"];
-        }
-
-        soft_clip_threshold = std::stoi(myIni["view_thresholds"]["soft_clip"]);
-        small_indel_threshold = std::stoi(myIni["view_thresholds"]["small_indel"]);
-        snp_threshold = std::stoi(myIni["view_thresholds"]["snp"]);
-        edge_highlights = std::stoi(myIni["view_thresholds"]["edge_highlights"]);
-        if (myIni["view_thresholds"].has("variant_distance")) {
-            variant_distance = std::stoi(myIni["view_thresholds"]["variant_distance"]);
-        }
-        if (myIni["view_thresholds"].has("low_memory")) {
-            low_memory = std::stoi(myIni["view_thresholds"]["low_memory"]);
-        }
-        if (myIni["view_thresholds"].has("mod_threshold")) {
-            mod_threshold = std::stoi(myIni["view_thresholds"]["mod_threshold"]);
+        indel_length = std::stoi(sub["indel_length"]);
+        ylim = std::stoi(sub["ylim"]);
+        split_view_size = std::stoi(sub["split_view_size"]);
+        threads = std::stoi(sub["threads"]);
+        pad = std::stoi(sub["pad"]);
+        max_coverage = (sub["coverage"] == "true") ? 100000 : 0;
+        log2_cov = sub["log2_cov"] == "true";
+        scroll_speed = std::stof(sub["scroll_speed"]);
+        tab_track_height = std::stof(sub["tabix_track_height"]);
+        if (sub.has("font")) {
+            font_str = sub["font"];
         } else {
-            myIni["view_thresholds"]["mod"] = "1000000";
+            sub["font"] = "Menlo";
+            update_ini = true;
+        }
+        if (sub.has("font_size")) {
+            font_size = std::stoi(sub["font_size"]);
+        } else {
+            sub["font_size"] = "14";
+            update_ini = true;
+        }
+        if (sub.has("expand_tracks")) {
+            expand_tracks = sub["expand_tracks"] == "true";
+        } else {
+            sub["expand_tracks"] = "true";
+            update_ini = true;
+        }
+        if (sub.has("scale_bar")) {
+            scale_bar = sub["scale_bar"] == "true";
+        } else {
+            sub["scale_bar"] = "true";
+            update_ini = true;
+        }
+        if (sub.has("sv_arcs")) {
+            sv_arcs = sub["sv_arcs"] == "true";
+        } else {
+            sub["sv_arcs"] = "true";
+            update_ini = true;
+        }
+        if (sub.has("mods")) {
+            parse_mods = sub["mods"] == "true";
+        } else {
+            sub["mods"] = "false";
+            update_ini = true;
+        }
+        if (sub.has("session_file")) {
+            session_file = sub["session_file"];
+        }  // defer update to saveCurrentSession
+
+        mINI::INIMap<std::string>& vt = myIni["view_thresholds"];
+
+        soft_clip_threshold = std::stoi(vt["soft_clip"]);
+        small_indel_threshold = std::stoi(vt["small_indel"]);
+        snp_threshold = std::stoi(vt["snp"]);
+        if (vt.has("edge_highlights")) {
+            edge_highlights = std::stoi(vt["edge_highlights"]);
+        } else {
+            vt["edge_highlights"] = "1000000";
+            update_ini = true;
+        }
+        if (vt.has("variant_distance")) {
+            variant_distance = std::stoi(vt["variant_distance"]);
+        } else {
+            vt["variant_distance"] = "100000";
+            update_ini = true;
+        }
+        if (vt.has("low_memory")) {
+            low_memory = std::stoi(vt["low_memory"]);
+        } else {
+            vt["low_memory"] = "1500000";
+            update_ini = true;
+        }
+        if (vt.has("mod_threshold")) {
+            mod_threshold = std::stoi(vt["mod_threshold"]);
+        } else {
+            vt["mod"] = "1000000";
         }
 
-        scroll_right = key_table[myIni["navigation"]["scroll_right"]];
-        scroll_left = key_table[myIni["navigation"]["scroll_left"]];
-        scroll_up = key_table[myIni["navigation"]["scroll_up"]];
-        scroll_down = key_table[myIni["navigation"]["scroll_down"]];
-        zoom_out = key_table[myIni["navigation"]["zoom_out"]];
-        zoom_in = key_table[myIni["navigation"]["zoom_in"]];
+        mINI::INIMap<std::string>& ng = myIni["navigation"];
 
-        cycle_link_mode = key_table[myIni["interaction"]["cycle_link_mode"]];
-        print_screen = key_table[myIni["interaction"]["print_screen"]];
-        if (myIni["interaction"].has("find_alignments")) {
-            find_alignments = key_table[myIni["interaction"]["find_alignments"]];
+        scroll_right = key_table[ng["scroll_right"]];
+        scroll_left = key_table[ng["scroll_left"]];
+        scroll_up = key_table[ng["scroll_up"]];
+        scroll_down = key_table[ng["scroll_down"]];
+        zoom_out = key_table[ng["zoom_out"]];
+        zoom_in = key_table[ng["zoom_in"]];
+
+        mINI::INIMap<std::string>& ia = myIni["interaction"];
+
+        cycle_link_mode = key_table[ia["cycle_link_mode"]];
+
+        if (ia.has("find_alignments")) {
+            find_alignments = key_table[ia["find_alignments"]];
+        } else {
+            ia["find_alignments"] = "F";
+            update_ini = true;
         }
-        if (myIni["interaction"].has("repeat_command")) {
-            repeat_command = key_table[myIni["interaction"]["repeat_command"]];
+        if (ia.has("repeat_command")) {
+            repeat_command = key_table[ia["repeat_command"]];
+        } else {
+            ia["repeat_command"] = "R";
+            update_ini = true;
         }
-        if (myIni["interaction"].has("vcf_as_tracks")) {
-            vcf_as_tracks = myIni["interaction"]["vcf_as_tracks"] == "true";
+        if (ia.has("vcf_as_tracks")) {
+            vcf_as_tracks = ia["vcf_as_tracks"] == "true";
+        } else {
+            ia["vcf_as_tracks"] = "false";
+            update_ini = true;
         }
-        if (myIni["interaction"].has("bed_as_tracks")) {
-            bed_as_tracks = myIni["interaction"]["bed_as_tracks"] == "true";
+        if (ia.has("bed_as_tracks")) {
+            bed_as_tracks = ia["bed_as_tracks"] == "true";
+        } else {
+            ia["bed_as_tracks"] = "true";
+            update_ini = true;
         }
 
-        number_str = myIni["labelling"]["number"];
+        mINI::INIMap<std::string>& lb = myIni["labelling"];
+
+        number_str = lb["number"];
         number = Utils::parseDimensions(number_str);
-        parse_label = myIni["labelling"]["parse_label"];
+        parse_label = lb["parse_label"];
 
-        labels = myIni["labelling"]["labels"];
-        delete_labels = key_table[myIni["labelling"]["delete_labels"]];
-        enter_interactive_mode = key_table[myIni["labelling"]["enter_interactive_mode"]];
+        labels = lb["labels"];
 
         if (myIni.has("shift_keymap")) {
-            shift_keymap["\\"] = "|";
-            shift_keymap[";"] = ":";
-            shift_keymap["["] = "{";
-            shift_keymap["]"] = "}";
-            shift_keymap["."] = ">";
-            shift_keymap[","] = "<";
-            shift_keymap["#"] = "~";
-            shift_keymap["-"] = "_";
+
+            mINI::INIMap<std::string>& km = myIni["shift_keymap"];
+
             shift_keymap["'"] = "@";
             shift_keymap["="] = "+";
-            shift_keymap["1"] = "!";
             shift_keymap["2"] = "\"";
             shift_keymap["3"] = "£";
-            shift_keymap["4"] = "$";
             shift_keymap["5"] = "%";
             shift_keymap["6"] = "^";
-            shift_keymap["7"] = "&";
             shift_keymap["8"] = "*";
             shift_keymap["9"] = "(";
             shift_keymap["0"] = ")";
             shift_keymap["`"] = "~";
+
+            if (km.has("ampersand")) {
+                shift_keymap[km["ampersand"]] = "&";
+            } else {
+                shift_keymap["7"] = "&";
+                update_ini = true;
+            }
+            if (km.has("bar")) {
+                shift_keymap[km["bar"]] = "|";
+            } else {
+                shift_keymap["\\"] = "|";
+                update_ini = true;
+            }
+            if (km.has("colon")) {
+                shift_keymap[km["colon"]] = ":";
+            } else {
+                shift_keymap[";"] = ":";
+                update_ini = true;
+            }
+            if (km.has("curly_open")) {
+                shift_keymap[km["curly_open"]] = "{";
+            } else {
+                shift_keymap["["] = "{";
+                update_ini = true;
+            }
+            if (km.has("curly_close")) {
+                shift_keymap[km["curly_close"]] = "{";
+            } else {
+                shift_keymap["]"] = "}";
+                update_ini = true;
+            }
+            if (km.has("dollar")) {
+                shift_keymap[km["dollar"]] = "$";
+            } else {
+                shift_keymap["4"] = "$";
+                update_ini = true;
+            }
+            if (km.has("exclamation")) {
+                shift_keymap[km["exclamation"]] = "!";
+            } else {
+                shift_keymap["1"] = "!";
+                update_ini = true;
+            }
+            if (km.has("greater_than")) {
+                shift_keymap[km["greater_than"]] = ">";
+            } else {
+                shift_keymap["."] = ">";
+                update_ini = true;
+            }
+            if (km.has("less_than")) {
+                shift_keymap[km["less_than"]] = "<";
+            } else {
+                shift_keymap[","] = "<";
+                update_ini = true;
+            }
+            if (km.has("tilde")) {
+                shift_keymap[km["tilde"]] = "~";
+            } else {
+                shift_keymap["`"] = "~";
+                update_ini = true;
+            }
+            if (km.has("underscore")) {
+                shift_keymap[km["underscore"]] = "_";
+            } else {
+                shift_keymap["-"] = "_";
+                update_ini = true;
+            }
+
+        }
+        if (update_ini) {
+            saveIniChanges();
         }
     }
 
@@ -578,8 +720,11 @@ namespace Themes {
             if (sub.has("expand_tracks")) {
                 expand_tracks = sub["expand_tracks"] == "true";
             }
+            if (sub.has("scale_bar")) {
+                scale_bar = sub["scale_bar"] == "true";
+            }
             if (sub.has("link")) {
-                link = sub["link"] == "true";
+                link = sub["link"];
                 link_op = 0;
                 if (link == "sv") {
                     link_op = 1;
@@ -607,6 +752,9 @@ namespace Themes {
             if (sub.has("mods_qual_threshold")) {
                 mods_qual_threshold = std::stoi(sub["mods_qual_threshold"]);
             }
+            if (sub.has("mods")) {
+                parse_mods = sub["mods"] == "true";
+            }
         }
         if (sesh.has("view_thresholds")) {
             mINI::INIMap<std::string>& vt = sesh["view_thresholds"];
@@ -689,6 +837,7 @@ namespace Themes {
                     if (path.empty()) {
                         std::cerr << "Error: .gw.ini file could not be read or created. Unexpected behavior may arise\n";
                         theme = Themes::DarkTheme();
+
                         return false;
                     }
                 }
@@ -703,7 +852,7 @@ namespace Themes {
 
     void IniOptions::saveIniChanges() {
         mINI::INIFile file(ini_path);
-        file.generate(myIni);
+        file.write(myIni, true);
     }
 
     void IniOptions::saveCurrentSession(std::string& genome_path, std::string& ideogram_path, std::vector<std::string>& bam_paths,
@@ -711,14 +860,14 @@ namespace Themes {
                                         std::vector<std::pair<std::string, int>>& variant_paths_info,
                                         std::vector<std::string>& commands, std::string output_session,
                                         int mode, int window_x_pos, int window_y_pos, float monitorScale,
-                                        int screen_width, int screen_height) {
+                                        int screen_width, int screen_height, int sortReadsBy) {
         if (output_session.empty()) {
 
             if (session_file.empty()) {  // fill new session
                 std::filesystem::path gwini(ini_path);
                 std::filesystem::path sesh(".gw_session.ini");
                 std::filesystem::path sesh_path = gwini.parent_path() / sesh;
-                myIni["general"].set("session_file", sesh_path.string());
+                myIni["general"]["session_file"] = sesh_path.string();
                 saveIniChanges();
             }
             output_session = myIni["general"]["session_file"];
@@ -738,7 +887,6 @@ namespace Themes {
         }
         count = 0;
         for (const auto& item : track_paths) {
-            std::cout << " track " << std::endl;
             seshIni["data"]["track" + std::to_string(count)] = item;
             count += 1;
         }
@@ -755,6 +903,15 @@ namespace Themes {
             count += 1;
         }
         seshIni["show"]["mode"] = (mode == 1) ? "tiled" : "single";
+
+        if (sortReadsBy == 0) {
+            seshIni["show"]["sort"] = "none";
+        } else if (sortReadsBy == 1) {
+            seshIni["show"]["sort"] = "strand";
+        } else if (sortReadsBy == 2) {
+            seshIni["show"]["sort"] = "hap";
+        }
+
         count = 0;
         for (auto& item: variant_paths_info) {
             seshIni["show"]["var" + std::to_string(count)] = std::to_string(item.second);
@@ -793,6 +950,7 @@ namespace Themes {
         sub["coverage"] = (max_coverage) ? "true" : "false";
         sub["log2_cov"] = (log2_cov) ? "true" : "false";
         sub["expand_tracks"] = (expand_tracks) ? "true" : "false";
+        sub["scale_bar"] = (scale_bar) ? "true" : "false";
         sub["mods"] = (parse_mods) ? "true" : "false";
         sub["link"] = link;
         sub["split_view_size"] = std::to_string(split_view_size);
@@ -882,36 +1040,38 @@ namespace Themes {
         }
     }
 
-    void printIdeogram(const std::unordered_map<std::string, std::vector<Band>> &bands) {
-        std::cout << "{\n";
-        for (const auto &kv: bands) {
-            std::cout << "{" << kv.first << ",\n{  \n";
-            for (const auto &b: kv.second) {
-                std::cout <<
-                          "  {" << b.start << ", "
-                          << b.end << ", "
-                          << b.alpha << ", "
-                          << b.red << ", "
-                          << b.green << ", "
-                          << b.blue << ", "
-                          << b.name << "},\n";
-            }
-            std::cout << "  },\n";
-        }
-        std::cout << "};\n\n";
-    }
+    #define next_t std::getline(iss, token, '\t')
 
-    void readIdeogramFile(std::string file_path, std::unordered_map<std::string, std::vector<Band>> &ideogram,
+    void readIdeogramFile(std::string file_path, std::unordered_map<std::string, std::vector<Ideo::Band>> &ideogram,
                           Themes::BaseTheme &theme) {
-        std::ifstream band_file(file_path);
-        if (!band_file) {
-            throw std::runtime_error("Failed to open input files");
+        std::shared_ptr<std::istream> fpu;
+#if !defined(__EMSCRIPTEN__)
+
+        if (Utils::startsWith(file_path, "http") || Utils::startsWith(file_path, "ftp")) {
+            std::string content = Utils::fetchOnlineFileContent(file_path);
+            fpu = std::make_shared<std::istringstream>(content);
+        } else {
+            auto file_stream = std::make_shared<std::ifstream>(file_path);
+            if (!file_stream->is_open()) {
+                std::cerr << "Error: opening ideogram file " << file_path << std::endl;
+                throw std::runtime_error("Error opening file");
+            }
+            fpu = file_stream;
         }
-        std::unordered_map<std::string, Band> custom;
+#else
+        fpu = std::make_shared<std::ifstream>();
+            fpu->open(p);
+            if (!fpu->is_open()) {
+                std::cerr << "Error: opening track file " << file_path << std::endl;
+                throw std::exception();
+            }
+#endif
+
+        std::unordered_map<std::string, Ideo::Band> custom;
         std::string line, token, chrom, name, property;
         int acen = theme.fcT.getColor();
         int gvar = theme.fcC.getColor();
-        while (std::getline(band_file, line)) {
+        while (std::getline(*fpu, line)) {
             std::istringstream iss(line);
             if (line[0] == '#') {
                 if (Utils::startsWith(line, "#gw ")) {
@@ -925,7 +1085,7 @@ namespace Themes {
                 }
                 continue;
             }
-            #define next_t std::getline(iss, token, '\t')
+
             next_t;
             chrom = token;
             next_t;
@@ -951,7 +1111,7 @@ namespace Themes {
             } else if (property == "gvar") {
                 ideogram[chrom].emplace_back() = {start, end, 255, SkColorGetR(gvar), SkColorGetG(gvar), SkColorGetB(gvar), {}, name};
             } else if (custom.find(name) != custom.end()) {
-                Band cust = custom[name];
+                Ideo::Band cust = custom[name];
                 cust.start = start;
                 cust.end = end;
                 ideogram[chrom].emplace_back() = cust;
@@ -976,6 +1136,88 @@ namespace Themes {
                 i.paint.setARGB(i.alpha, i.red, i.green, i.blue);
             }
         }
-//        printIdeogram(ideogram);
     }
+
+
+    void readIdeogramData(const unsigned char *data, size_t size, std::unordered_map<std::string, std::vector<Ideo::Band>> &ideogram,
+                          Themes::BaseTheme &theme, bool strip_chr) {
+        if (size == 0) {
+            return;
+        }
+        std::istringstream fpu(std::string(reinterpret_cast<const char*>(data), size));
+
+        std::unordered_map<std::string, Ideo::Band> custom;
+        std::string line, token, chrom, name, property;
+        int acen = theme.fcT.getColor();
+        int gvar = theme.fcC.getColor();
+        while (std::getline(fpu, line)) {
+            std::istringstream iss(line);
+            if (line[0] == '#') {
+                if (Utils::startsWith(line, "#gw ")) {
+                    std::vector<std::string> parts = Utils::split(line, ' ');
+                    if (parts.size() == 3) {
+                        std::vector<std::string> c = Utils::split(parts.back(), ',');
+                        if (c.size() == 4) {
+                            custom[parts[1]] = {0, 0, std::stoi(c[0]), std::stoi(c[1]), std::stoi(c[2]), std::stoi(c[3]), {}, parts[1]};
+                        }
+                    }
+                }
+                continue;
+            }
+
+            next_t;
+            chrom = token;
+            if (strip_chr) {
+                chrom.erase(chrom.begin(), chrom.begin() + 3);
+            }
+            next_t;
+            int start = std::stoi(token);
+            next_t;
+            int end = std::stoi(token);
+            next_t;
+            name = token;
+            next_t;
+            property = token;
+            if (property == "gneg") {
+                ideogram[chrom].emplace_back() = {start, end, 255, 255, 255, 255, {}, name};
+            } else if (property == "gpos25") {
+                ideogram[chrom].emplace_back() = {start, end, 255, 235, 235, 235, {}, name};
+            } else if (property == "gpos50") {
+                ideogram[chrom].emplace_back() = {start, end, 255, 185, 185, 185, {}, name};
+            } else if (property == "gpos75") {
+                ideogram[chrom].emplace_back() = {start, end, 255, 110, 110, 110, {}, name};
+            } else if (property == "gpos100") {
+                ideogram[chrom].emplace_back() = {start, end, 255, 60, 60, 60, {}, name};
+            } else if (property == "acen") {
+                ideogram[chrom].emplace_back() = {start, end, 255, SkColorGetR(acen), SkColorGetG(acen), SkColorGetB(acen), {}, name};
+            } else if (property == "gvar") {
+                ideogram[chrom].emplace_back() = {start, end, 255, SkColorGetR(gvar), SkColorGetG(gvar), SkColorGetB(gvar), {}, name};
+            } else if (custom.find(name) != custom.end()) {
+                Ideo::Band cust = custom[name];
+                cust.start = start;
+                cust.end = end;
+                ideogram[chrom].emplace_back() = cust;
+            } else if (!property.empty()) {  // try custom color scheme
+                std::vector<std::string> a = Utils::split(property, ',');
+                if (a.size() == 4) {
+                    try {
+                        ideogram[chrom].emplace_back() = {start, end, std::stoi(a[0]), std::stoi(a[1]), std::stoi(a[2]),
+                                                          std::stoi(a[3]), {}, name};
+                    } catch (...) {
+
+                    }
+                } else {
+                    ideogram[chrom].emplace_back() = {start, end, 255, 85, 171, 159, {}, name};
+                }
+            } else {
+                ideogram[chrom].emplace_back() = {start, end, 255, 85, 171, 159, {}, name};
+            }
+        }
+        for (auto& kv : ideogram) {
+            for (auto& i : kv.second) {
+                i.paint.setARGB(i.alpha, i.red, i.green, i.blue);
+            }
+        }
+    }
+
 }
